import {
  histogram as bin,
  scaleLinear,
  scaleTime,
  timeMonths,
  timeFormat,
  max,
  extent,
  sum,
  brushX,
  select,
  event,
} from 'd3';
import { useRef, useEffect, useMemo } from 'react';
import { AxisBottom } from './AxisBottom'; // ./import from current directory
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';

const margin = {
  top: 0,
  right: 30,
  bottom: 20,
  left: 50,
};
const xAxisLabelOffset = 54;
const yAxisLabelOffset = 30;
const xAxisLabel = 'Reported Date';
const xAxisTickFormat = timeFormat('%m/%d/%Y');
const yValue = (d) => d['Total Dead and Missing'];
const yAxisLabel = 'Total Dead and Missing';

export const DateHistogram = ({
  data,
  width,
  height,
  setBrushExtent,
  xValue,
}) => {
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  const xScale = useMemo(
    () =>
      scaleTime()
        .domain(extent(data, xValue)) // Returns array with min and max for column
        .range([0, innerWidth])
        .nice(),
    [data, xValue, innerWidth] // dependency array when to recalculate xAxis
  );

  // Aggregate data
  // bin is a generator. Thats why we need (data) at the end
  const binnedData = useMemo(() => {
    const [start, stop] = xScale.domain();
    return (
      bin()
        .value(xValue)
        .domain(xScale.domain())
        .thresholds(timeMonths(start, stop))(data)
        // Take grouped data and sum over it. Copy x0, x1
        .map((array) => ({
          y: sum(array, yValue),
          x0: array.x0,
          x1: array.x1,
        }))
    );
  }, [xValue, yValue, xScale, data]);

  const yScale = useMemo(
    () =>
      scaleLinear()
        .domain([0, max(binnedData, (d) => d.y)])
        .range([innerHeight, 0]),
    [binnedData, innerHeight]
  );

  // Setup interactive Brush
  const brushRef = useRef();
  useEffect(() => {
    const brush = brushX().extent([
      [0, 0],
      [innerWidth, innerHeight],
    ]);
    brush(select(brushRef.current));
    // EventListener and Callback Function
    brush.on('brush end', () => {
      // In order to change the state in other code, we use the state hook, which was passed as a functin to this component
      setBrushExtent(
        event.selection &&
          event.selection.map(xScale.invert)
      ); // invert takes input from the range (here pixel coordinates) of the scale and returns the date. We use map to cast the method on the array (left and right coordinate).
    });
  }, [innerWidth, innerHeight]); // dependency array. Each time they change, the effect is executed again!

  // Return JSX component
  return (
    <>
      <rect
        x={0}
        y={0}
        width={width}
        height={height}
        fill="white"
      />
      <g
        transform={`translate(${margin.left},${margin.top})`}
      >
        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
          tickOffset={7}
        />
        <text
          className="axis-label"
          textAnchor="middle"
          transform={`translate(${-yAxisLabelOffset},${
            innerHeight / 2
          }) rotate(-90)`} // In order to rotate around text we need to translate it (not set x and y and then afterwards call rotate)
        >
          {yAxisLabel}
        </text>
        <AxisLeft
          yScale={yScale}
          innerWidth={innerWidth}
          tickOffset={7}
        />
        <text
          className="axis-label"
          x={innerWidth / 2}
          y={innerHeight + xAxisLabelOffset}
          textAnchor="middle"
        >
          {xAxisLabel}
        </text>
        <Marks
          // Need to x/yValue functions, since binnedData has x0,x1,y
          binnedData={binnedData}
          xScale={xScale}
          yScale={yScale}
          innerHeight={innerHeight}
          tooltipFormat={(d) => d}
        />
        // Needs to be at the bottom.
        <g ref={brushRef} />
      </g>
    </>
  );
};
